<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns="urn:WebWorks-Document-Schema" 
                              xmlns:wwdoc="urn:WebWorks-Document-Schema" 
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
>
 <xsl:output method="xml" encoding="utf-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="wwdoc" result-prefix="#default" />
 <xsl:strip-space elements="*" />
 
 <xsl:template match="/">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:Document version="1.1">
   <wwdoc:Styles>
    <wwdoc:ParagraphStyles>
     <wwdoc:ParagraphStyle name="rst-section-1" />
     <wwdoc:ParagraphStyle name="rst-section-2" />
     <wwdoc:ParagraphStyle name="rst-section-3" />
     <wwdoc:ParagraphStyle name="rst-section-4" />
     <wwdoc:ParagraphStyle name="rst-section-5" />
     <wwdoc:ParagraphStyle name="rst-paragraph" />
     <wwdoc:ParagraphStyle name="rst-paragraph-indent-1" />
     <wwdoc:ParagraphStyle name="rst-paragraph-indent-2" />
     <wwdoc:ParagraphStyle name="rst-paragraph-indent-3" />
     <wwdoc:ParagraphStyle name="rst-paragraph-numbered-1" />
     <wwdoc:ParagraphStyle name="rst-paragraph-bulleted-1" />
     <wwdoc:ParagraphStyle name="rst-paragraph-cell-head" />
     <wwdoc:ParagraphStyle name="rst-paragraph-cell-body" />
     <wwdoc:ParagraphStyle name="rst-paragraph-cell-foot" />
     <wwdoc:ParagraphStyle name="rst-literal-block" />
    </wwdoc:ParagraphStyles>
    
    <wwdoc:CharacterStyles>
     <wwdoc:CharacterStyle name="rst-literal" />
    </wwdoc:CharacterStyles>
    
    <wwdoc:TableStyles>
     <wwdoc:TableStyle name="rst1" />
    </wwdoc:TableStyles>
   </wwdoc:Styles>
   <wwdoc:Content>
    <xsl:apply-templates select="$ParamContext/document" />
   </wwdoc:Content>
  </wwdoc:Document>
 </xsl:template>
 
 <xsl:template match="document">
  <xsl:param name="ParamContext" select="." />

  <xsl:apply-templates select="$ParamContext/section" />
 </xsl:template>
 
 <xsl:template match="section">
  <xsl:param  name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>p</xsl:text>
   <xsl:number level="multiple" count="section" format="a.a" />
  </xsl:variable>
  
  <xsl:variable name="VarStyleName">
   <xsl:text>rst-section-</xsl:text>
   <xsl:value-of select="count(ancestor-or-self::section)" />
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="{$VarStyleName}">
   <xsl:apply-templates select="$ParamContext/title" mode="section" />
  </wwdoc:Paragraph>

  <xsl:apply-templates select="$ParamContext/*[local-name() != 'section']" />
  <xsl:apply-templates select="$ParamContext/section" />
 </xsl:template>
 
 <xsl:template match="title">
  <!-- no op -->
 </xsl:template>
 
 <xsl:template match="title" mode="section">
  <xsl:param name="ParamContext" select="." />

  <wwdoc:Number value="{substring-before($ParamContext/generated/text(), '&#xA0;')}">
   <wwdoc:Text value="{normalize-space($ParamContext/generated/text())}" />
  </wwdoc:Number>
  <wwdoc:TextRun>
   <wwdoc:Text value="{$ParamContext/text()}" />
  </wwdoc:TextRun>
 </xsl:template>
 
 <xsl:template match="paragraph">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>p</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <xsl:variable name="VarStyleName">
   <xsl:variable name="VarBlockquoteAncestors" select="ancestor::block_quote" />
   <xsl:variable name="VarEnumeratedAncestors" select="ancestor::enumerated_list" />
   <xsl:variable name="VarBulletAncestors" select="ancestor::bullet_list" />
   <xsl:variable name="VarTableHeadAncestors" select="ancestor::thead" />
   <xsl:variable name="VarTableBodyAncestors" select="ancestor::tbody" />
   <xsl:variable name="VarTableFootAncestors" select="ancestor::tfoot" />
  
   <xsl:text>rst-paragraph</xsl:text>
   
   <xsl:choose>
    <xsl:when test="count($VarTableHeadAncestors[1]) = 1">
     <xsl:text>-cell-head</xsl:text>
    </xsl:when>

    <xsl:when test="count($VarTableBodyAncestors[1]) = 1">
     <xsl:text>-cell-body</xsl:text>
    </xsl:when>

    <xsl:when test="count($VarTableFootAncestors[1]) = 1">
     <xsl:text>-cell-foot</xsl:text>
    </xsl:when>

    <xsl:when test="count($VarBlockquoteAncestors[1]) = 1">
     <xsl:text>-indent-</xsl:text>
     <xsl:value-of select="count($VarBlockquoteAncestors)" />
    </xsl:when>

    <xsl:when test="count($VarEnumeratedAncestors[1]) = 1">
     <xsl:text>-numbered-</xsl:text>
     <xsl:value-of select="count($VarEnumeratedAncestors)" />
    </xsl:when>

    <xsl:when test="count($VarBulletAncestors[1]) = 1">
     <xsl:text>-bulleted-</xsl:text>
     <xsl:value-of select="count($VarBulletAncestors)" />
    </xsl:when>
   </xsl:choose>
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="{$VarStyleName}">
  
   <xsl:if test="count(parent::list_item[1]) = 1">
    <xsl:variable name="VarNumberValue" select="count(parent::list_item[1]/preceding-sibling::list_item) + 1" />

    <wwdoc:Number value="{$VarNumberValue}">
     <wwdoc:Text value="{$VarNumberValue}.&#x00A0;" />
    </wwdoc:Number>
   </xsl:if>

   <xsl:for-each select="text() | node()[string-length(local-name()) &gt; 0]">
    <xsl:apply-templates select="." mode="character" />
   </xsl:for-each>
  </wwdoc:Paragraph>
 </xsl:template>
 
 <xsl:template match="literal_block">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>p</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="rst-literal-block">
   <xsl:apply-templates select="text()" mode="preserve-space" />
  </wwdoc:Paragraph>
 </xsl:template>

 <xsl:template match="block_quote">
  <xsl:param name="ParamContext" select="." />
  
  <xsl:apply-templates select="$ParamContext/*" />
 </xsl:template>

 <xsl:template match="table">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>t</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <wwdoc:Table id="{$VarId}" stylename="rst1">
   <wwdoc:Style>
    <wwdoc:Attribute name="text-align" value="left" />
    <wwdoc:TableHead>
     <wwdoc:Style />
    </wwdoc:TableHead>
    <wwdoc:TableBody>
     <wwdoc:Style />
    </wwdoc:TableBody>
    <wwdoc:TableFoot>
     <wwdoc:Style />
    </wwdoc:TableFoot>
   </wwdoc:Style>
 
   <xsl:apply-templates select="$ParamContext/*" />
  </wwdoc:Table>
 </xsl:template>
 
 <xsl:template match="tgroup">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableColumns value="{$ParamContext/@cols}">
   <xsl:for-each select="$ParamContext/colspec">
    <wwdoc:TableColumn width="{@colwidth}" />
   </xsl:for-each>
  </wwdoc:TableColumns>
  
  <wwdoc:Style>
   <wwdoc:Attribute name="width" value="" />
  </wwdoc:Style>
  
  <wwdoc:Caption />
  
  <xsl:apply-templates select="$ParamContext/thead | $ParamContext/tbody | $ParamContext/tfoot" />
 </xsl:template>
 
 <xsl:template match="thead">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableHead>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableHead>
 </xsl:template>
 
 <xsl:template match="tbody">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableBody>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableBody>
 </xsl:template>
 
 <xsl:template match="tfoot">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableFoot>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableFoot>
 </xsl:template>
 
 <xsl:template match="row">
  <xsl:param name="ParamContext" select="." />
  <xsl:param name="ParamPosition" select="position()" />
  
  <wwdoc:TableRow>
   <xsl:apply-templates select="$ParamContext/entry">
    <xsl:with-param name="ParamRowPosition" select="$ParamPosition" />
   </xsl:apply-templates>
  </wwdoc:TableRow>
 </xsl:template>
 
 <xsl:template match="entry">
  <xsl:param name="ParamContext" select="." />
  <xsl:param name="ParamRowPosition" select="1" />

  <xsl:variable name="VarBackgroundColor">
   <xsl:choose>
    <xsl:when test="count(ancestor::thead[1]) = 1">
     <xsl:value-of select="'lightgreen'" />
    </xsl:when>

    <xsl:when test="($ParamRowPosition mod 2) = 0">
     <xsl:value-of select="'silver'" />
    </xsl:when>
   </xsl:choose>
  </xsl:variable>

  <wwdoc:TableCell>
   <xsl:if test="string-length($VarBackgroundColor) &gt; 0">
    <wwdoc:Style>
     <wwdoc:Attribute name="background-color" value="{$VarBackgroundColor}" />
    </wwdoc:Style>
   </xsl:if>

   <xsl:apply-templates select="$ParamContext/*" />
  </wwdoc:TableCell>
 </xsl:template>

 <xsl:template match="node()" mode="character">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>tr-</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
   <xsl:text>.</xsl:text>
   <xsl:value-of select="position()" />
  </xsl:variable>

  <wwdoc:TextRun id="{$VarId}">
   <xsl:if test="string-length(local-name($ParamContext)) &gt; 0">
    <xsl:attribute name="stylename">
     <xsl:value-of select="concat('rst-', local-name($ParamContext))" />
    </xsl:attribute>
   </xsl:if>
   <wwdoc:Text value="{$ParamContext | $ParamContext/text()}" />
  </wwdoc:TextRun>
 </xsl:template>
 
 <xsl:template name="Preserve-Space" match="node()" mode="preserve-space">
  <xsl:param name="ParamContext" select="." />
  
  <xsl:variable name="VarId">
   <xsl:text>tr-</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>
  
  <xsl:choose>
  <xsl:when test="contains($ParamContext, '&#xD;&#xA;')">
   <wwdoc:TextRun id="{$VarId}">
    <wwdoc:Text value="{substring-before($ParamContext, '&#xD;&#xA;')}" />
   </wwdoc:TextRun>
   <wwdoc:LineBreak />
   
   <xsl:call-template name="Preserve-Space">
    <xsl:with-param name="ParamContext" select="substring-after($ParamContext, '&#xD;&#xA;')" />
   </xsl:call-template>
  </xsl:when>

  <xsl:otherwise>
   <wwdoc:TextRun id="{$VarId}">
    <wwdoc:Text value="{$ParamContext}" />
   </wwdoc:TextRun>
  </xsl:otherwise>
  </xsl:choose>
 </xsl:template>
</xsl:stylesheet>